@@ -130,9 +130,13 @@ shinyAppsClient <- function(service, authInfo) {
       GET(service, authInfo, path, paste(m, queryString(query), sep = "&"))
     },
 
-    getLogs = function(applicationId, entries = 50) {
+    getLogs = function(applicationId, entries = 50, format = NULL) {
       path <- paste0("/applications/", applicationId, "/logs")
       query <- paste0("count=", entries, "&tail=0")
+      if (!is.null(format)) {
+        # format=json returns a structured response.
+        query <- paste0(query, "&format=", format)
+      }
       GET(service, authInfo, path, query)
     },
 
@@ -142,8 +146,7 @@ shinyAppsClient <- function(service, authInfo) {
       template,
       accountId,
       appMode,
-      contentCategory = NULL,
-      spaceId = NULL
+      contentCategory = NULL
     ) {
       json <- list()
       json$name <- name
@@ -209,7 +212,7 @@ shinyAppsClient <- function(service, authInfo) {
       )
     },
 
-    deployApplication = function(application, bundleId = NULL, spaceId = NULL) {
+    deployApplication = function(application, bundleId = NULL) {
       path <- paste("/applications/", application$id, "/deploy", sep = "")
       json <- list()
       if (length(bundleId) > 0 && nzchar(bundleId)) {
